using DressAtelierBusinessLogic.BusinessLogic;
using DressAtelierContracts.BusinessLogicContracts;
using DressAtelierContracts.StorageContracts;
using DressAtelierFileImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using System;

namespace SewingDresses
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;

        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IMaterialStorage, MaterialStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IDressStorage, DressStorage>();
            services.AddTransient<IMaterialLogic, MaterialLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IDressLogic, DressLogic>();
            services.AddTransient<FormMain>();
            services.AddTransient<FormMaterial>();
            services.AddTransient<FormMaterials>();
            services.AddTransient<FormOrderCreation>();
            services.AddTransient<FormDress>();
            services.AddTransient<FormDressMaterial>();
            services.AddTransient<FormDresses>();
        }

    }
}