﻿using DressAtelierFileImplement.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace DressAtelierFileImplement
{
    public class DataFileSingleton
    {
        private static DataFileSingleton? instance;
        private readonly string MaterialFileName = "Material.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string DressFileName = "Dress.xml";
        public List<Material> Components { get; private set; }
        public List<Order> Orders { get; private set; }
        public List<Dress> Dresses { get; private set; }
        public static DataFileSingleton GetInstance()
        {
            if (instance == null)
            {
                instance = new DataFileSingleton();
            }
            return instance;
        }
        public void SaveComponents() => SaveData(Components, MaterialFileName,"Components", x => x.GetXElement);
        public void SaveDresses() => SaveData(Dresses, DressFileName,"Dresses", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName,"Orders", x => x.GetXElement);
        private DataFileSingleton()
        {
            Components = LoadData(MaterialFileName, "Component", x => Material.Create(x)!)!;
            Dresses = LoadData(DressFileName, "Dress", x => Dress.Create(x)!)!;
            Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName,
       Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return
               XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }
        private static void SaveData<T>(List<T> data, string filename, string
       xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName,
               data.Select(selectFunction).ToArray())).Save(filename);
            }
        }

    }
}
