﻿using DressAtelierContracts.BindingModels;
using DressAtelierContracts.ViewModels;
using DressAtelierDataModels.Enums;
using DressAtelierDataModels.Models;
using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace DressAtelierFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int ID { get; private set; }
        public int DressID { get; private set; }

        public int Count { get; private set; }

        public double Sum { get; private set; }

        public OrderStatus Status { get; private set; } = OrderStatus.Unknown;

        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                ID = model.ID,
                DressID = model.DressID,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };

        }

        public static Order? Create(XElement element)
        {
            if(element == null)
            {
                return null;
            }
            return new Order()
            {
                ID = Convert.ToInt32(element.Attribute("ID")!.Value),
                DressID = Convert.ToInt32(element.Element("DressID")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate = Convert.ToDateTime(element.Element("CreationDate")!.Value),
                DateImplement = string.IsNullOrEmpty(element.Element("ImplementationDate")!.Value) ? null : Convert.ToDateTime(element.Element("ImplementationDate")!.Value)
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            DressID = model.DressID;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            ID = ID,
            DressID = DressID,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement

        };

        public XElement GetXElement => new("Order", new XAttribute("ID", ID),
                                                       new XElement("DressID", DressID),
                                                       new XElement("Count", Count),
                                                       new XElement("Sum", Sum),
                                                       new XElement("Status", Status),
                                                       new XElement("CreationDate", DateCreate),
                                                       new XElement("ImplementationDate", DateImplement));
    }
}
